import { FlagEnum } from '../utils.js'

export const ShaderStage = FlagEnum(
  'vertex',
  'fragment',
  'compute'
)

/**
 * @param {GPUShaderStageFlags} stages
 * @returns {string[]}
 */
export const stageFlagToName = stages => {
  const names = []

  if (stages & GPUShaderStage.FRAGMENT) {
    names.push('fragment')
  }

  if (stages & GPUShaderStage.VERTEX) {
    names.push('vertex')
  }

  if (stages & GPUShaderStage.COMPUTE) {
    names.push('compute')
  }

  return names
}

/**
 * @param {('fragment' | 'vertex' | 'compute')[]} names
 */
export const nameToStageFlag = names => {
  return names.reduce((flags, name) => {
    switch (name.toLowerCase()) {
      case 'fragment': return flags | GPUShaderStage.FRAGMENT
      case 'vertex': return flags | GPUShaderStage.VERTEX
      case 'compute': return flags | GPUShaderStage.COMPUTE
    }
  }, 0)
}

export const ResourceType = Object.freeze({
  Sampler: 0,
  TextureView: 1,
  Buffer: 2,
  ExternalTexture: 3
})

